@@ -23,6 +23,7 @@
 #include "application.h"
 
 #include "extractor.h"
+#include "filemanager.h"
 #include "networkaccessmanager.h"
 #include "settings.h"
 
@@ -61,6 +62,8 @@ Application::Application(QObject *parent) :
     m_settings = new Settings(this);
     m_networkManager = new NetworkAccessManager(this);
 
+    m_fileManager = new FileManager(this);
+
     // Extractor setup
     m_extractorThread = new QThread(this);
     m_extractor = new Extractor();
@@ -106,6 +109,11 @@ Application *Application::instance()
     return m_instance;
 }
 
+WidgetUi::MainWindow *Application::mainWindow() const
+{
+    return m_mainWindow;
+}
+
 QNetworkAccessManager *Application::networkManager() const
 {
     return m_networkManager;
@@ -121,6 +129,11 @@ Registry::DocsetRegistry *Application::docsetRegistry()
     return m_docsetRegistry;
 }
 
+FileManager *Application::fileManager() const
+{
+    return m_fileManager;
+}
+
 void Application::executeQuery(const Registry::SearchQuery &query, bool preventActivation)
 {
     m_mainWindow->search(query);
