@@ -132,7 +132,8 @@ Docset::Docset(const QString &path) :
     sqlite3_create_function(m_db->handle(), "zealScore", 2, SQLITE_UTF8, nullptr,
                             sqliteScoreFunction, nullptr, nullptr);
 
-    m_type = m_db->tables().contains(QStringLiteral("searchIndex")) ? Type::Dash : Type::ZDash;
+    m_type = m_db->tables().contains(QStringLiteral("searchIndex"), Qt::CaseInsensitive)
+            ? Type::Dash : Type::ZDash;
 
     createIndex();
 
@@ -157,8 +158,9 @@ Docset::Docset(const QString &path) :
 
     if (plist.contains(InfoPlist::DashDocSetFamily)) {
         const QString kw = plist[InfoPlist::DashDocSetFamily].toString();
-        if (kw != QLatin1String("dashtoc") && kw != QLatin1String("unsorteddashtoc"))
+        if (!kw.contains(QLatin1String("dashtoc"))) {
             m_keywords << kw;
+        }
     }
 
     m_keywords.removeDuplicates();
@@ -211,6 +213,11 @@ QString Docset::revision() const
     return m_revision;
 }
 
+QString Docset::feedUrl() const
+{
+    return m_feedUrl;
+}
+
 QString Docset::path() const
 {
     return m_path;
@@ -265,7 +272,7 @@ QList<SearchResult> Docset::search(const QString &query, const CancellationToken
                                  "  FROM searchIndex"
                                  "  WHERE score > 0");
         } else {
-            sql = QStringLiteral("SELECT name, type, path, ''"
+            sql = QStringLiteral("SELECT name, type, path, '', -length(name) as score"
                                  "  FROM searchIndex"
                                  "  WHERE (name LIKE '%%1%' ESCAPE '\\')");
         }
@@ -275,7 +282,7 @@ QList<SearchResult> Docset::search(const QString &query, const CancellationToken
                                  "  FROM searchIndex"
                                  "  WHERE score > 0");
         } else {
-            sql = QStringLiteral("SELECT name, type, path, fragment"
+            sql = QStringLiteral("SELECT name, type, path, fragment, -length(name) as score"
                                  "  FROM searchIndex"
                                  "  WHERE (name LIKE '%%1%' ESCAPE '\\')");
         }
@@ -371,6 +378,10 @@ void Docset::loadMetadata()
     m_version = jsonObject[QStringLiteral("version")].toString();
     m_revision = jsonObject[QStringLiteral("revision")].toString();
 
+    if (jsonObject.contains(QStringLiteral("feed_url"))) {
+        m_feedUrl = jsonObject[QStringLiteral("feed_url")].toString();
+    }
+
     if (jsonObject.contains(QStringLiteral("extra"))) {
         const QJsonObject extra = jsonObject[QStringLiteral("extra")].toObject();
 
@@ -509,7 +520,8 @@ QUrl Docset::createPageUrl(const QString &path, const QString &fragment) const
     realPath.remove(dashEntryRegExp);
     realFragment.remove(dashEntryRegExp);
 
-    QUrl url = QUrl::fromLocalFile(QDir(documentPath()).filePath(realPath));
+    // Absolute file path is required here to handle relative path to the docset storage (see #806).
+    QUrl url = QUrl::fromLocalFile(QDir(documentPath()).absoluteFilePath(realPath));
     if (!realFragment.isEmpty()) {
         if (realFragment.startsWith(QLatin1String("//apple_ref"))
                 || realFragment.startsWith(QLatin1String("//dash_ref"))) {
