@@ -28,6 +28,7 @@
 #include "progressitemdelegate.h"
 
 #include <core/application.h>
+#include <core/filemanager.h>
 #include <core/settings.h>
 #include <registry/docset.h>
 #include <registry/docsetregistry.h>
@@ -35,20 +36,24 @@
 #include <registry/listmodel.h>
 
 #include <QClipboard>
+#include <QDateTime>
 #include <QDir>
-#include <QFutureWatcher>
 #include <QInputDialog>
+#include <QJsonArray>
+#include <QJsonDocument>
+#include <QMessageBox>
 #include <QNetworkReply>
 #include <QNetworkRequest>
-#include <QMessageBox>
 #include <QTemporaryFile>
 #include <QUrl>
 
-#include <QtConcurrent/QtConcurrent>
-
 using namespace Zeal;
 using namespace Zeal::WidgetUi;
 
+#ifdef Q_OS_WIN32
+extern Q_CORE_EXPORT int qt_ntfs_permission_lookup;
+#endif
+
 namespace {
 const char ApiServerUrl[] = "http://api.zealdocs.org/v1";
 const char RedirectServerUrl[] = "https://go.zealdocs.org";
@@ -71,12 +76,21 @@ DocsetsDialog::DocsetsDialog(Core::Application *app, QWidget *parent) :
     m_application(app),
     m_docsetRegistry(app->docsetRegistry())
 {
-    using Registry::DocsetRegistry;
-    using Registry::ListModel;
-
     ui->setupUi(this);
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
+    loadDocsetList();
+
+#ifdef Q_OS_WIN32
+    qt_ntfs_permission_lookup++;
+#endif
+
+    m_isStorageReadOnly = !QFileInfo(m_application->settings()->docsetPath).isWritable();
+
+#ifdef Q_OS_WIN32
+    qt_ntfs_permission_lookup--;
+#endif
+
 #ifdef Q_OS_OSX
     ui->availableDocsetList->setAttribute(Qt::WA_MacShowFocusRect, false);
     ui->installedDocsetList->setAttribute(Qt::WA_MacShowFocusRect, false);
@@ -84,90 +98,7 @@ DocsetsDialog::DocsetsDialog(Core::Application *app, QWidget *parent) :
 
     ui->combinedProgressBar->hide();
     ui->cancelButton->hide();
-
-    // Installed docsets tab
-    ui->installedDocsetList->setItemDelegate(new DocsetListItemDelegate(this));
-    ui->installedDocsetList->setModel(new ListModel(app->docsetRegistry(), this));
-    connect(ui->installedDocsetList, &QListView::activated, this, [this](const QModelIndex &index) {
-        if (!index.data(Registry::ItemDataRole::UpdateAvailableRole).toBool()) {
-            return;
-        }
-
-        downloadDashDocset(index);
-    });
-
-    QItemSelectionModel *selectionModel = ui->installedDocsetList->selectionModel();
-    connect(selectionModel, &QItemSelectionModel::selectionChanged,
-            [this, selectionModel]() {
-        ui->removeDocsetsButton->setEnabled(selectionModel->hasSelection());
-
-        for (const QModelIndex &index : selectionModel->selectedRows()) {
-            if (index.data(Registry::ItemDataRole::UpdateAvailableRole).toBool()) {
-                ui->updateSelectedDocsetsButton->setEnabled(true);
-                return;
-            }
-        }
-
-        ui->updateSelectedDocsetsButton->setEnabled(false);
-    });
-    connect(ui->updateSelectedDocsetsButton, &QPushButton::clicked,
-            this, &DocsetsDialog::updateSelectedDocsets);
-    connect(ui->updateAllDocsetsButton, &QPushButton::clicked,
-            this, &DocsetsDialog::updateAllDocsets);
-    connect(ui->removeDocsetsButton, &QPushButton::clicked,
-            this, &DocsetsDialog::removeSelectedDocsets);
-    connect(ui->docsetFilterInput, &QLineEdit::textEdited,
-            this, &DocsetsDialog::updateDocsetFilter);
-
-    // Available docsets tab
-    ui->availableDocsetList->setItemDelegate(new ProgressItemDelegate(this));
-    connect(ui->availableDocsetList, &QListView::activated, this, [this](const QModelIndex &index) {
-        // TODO: Cancel download if it's already in progress.
-        if (index.data(ProgressItemDelegate::ShowProgressRole).toBool())
-            return;
-
-        ui->availableDocsetList->selectionModel()->select(index, QItemSelectionModel::Deselect);
-
-        QAbstractItemModel *model = ui->availableDocsetList->model();
-        model->setData(index, tr("Downloading: %p%"), ProgressItemDelegate::FormatRole);
-        model->setData(index, 0, ProgressItemDelegate::ValueRole);
-        model->setData(index, true, ProgressItemDelegate::ShowProgressRole);
-
-        downloadDashDocset(index);
-    });
-
-    selectionModel = ui->availableDocsetList->selectionModel();
-    connect(selectionModel, &QItemSelectionModel::selectionChanged, [this, selectionModel]() {
-        for (const QModelIndex &index : selectionModel->selectedRows()) {
-            if (!index.data(ProgressItemDelegate::ShowProgressRole).toBool()) {
-                ui->downloadDocsetsButton->setEnabled(true);
-                return;
-            }
-        }
-
-        ui->downloadDocsetsButton->setEnabled(false);
-    });
-
-    connect(ui->downloadDocsetsButton, &QPushButton::clicked,
-            this, &DocsetsDialog::downloadSelectedDocsets);
-
-    connect(m_docsetRegistry, &DocsetRegistry::docsetRemoved, this, [this](const QString name) {
-        QListWidgetItem *item = findDocsetListItem(name);
-        if (!item)
-            return;
-
-        item->setHidden(false);
-    });
-    connect(m_docsetRegistry, &DocsetRegistry::docsetAdded, this, [this](const QString name) {
-        QListWidgetItem *item = findDocsetListItem(name);
-        if (!item)
-            return;
-
-        item->setHidden(true);
-    });
-
-    connect(ui->addFeedButton, &QPushButton::clicked, this, &DocsetsDialog::addDashFeed);
-    connect(ui->refreshButton, &QPushButton::clicked, this, &DocsetsDialog::downloadDocsetList);
+    ui->readOnlyLabel->setVisible(m_isStorageReadOnly);
 
     connect(m_application, &Core::Application::extractionCompleted,
             this, &DocsetsDialog::extractionCompleted);
@@ -178,7 +109,15 @@ DocsetsDialog::DocsetsDialog(Core::Application *app, QWidget *parent) :
 
     connect(ui->cancelButton, &QPushButton::clicked, this, &DocsetsDialog::cancelDownloads);
 
-    loadDocsetList();
+    setupInstalledDocsetsTab();
+    setupAvailableDocsetsTab();
+
+    if (m_isStorageReadOnly) {
+        disableControls();
+
+        // Updating the docset list is fine;
+        ui->refreshButton->setEnabled(true);
+    }
 }
 
 DocsetsDialog::~DocsetsDialog()
@@ -188,7 +127,7 @@ DocsetsDialog::~DocsetsDialog()
 
 void DocsetsDialog::reject()
 {
-    if (m_replies.isEmpty() && m_tmpFiles.isEmpty() && m_docsetsBeingDeleted.isEmpty()) {
+    if (m_replies.isEmpty() && m_tmpFiles.isEmpty()) {
         QDialog::reject();
         return;
     }
@@ -406,20 +345,37 @@ void DocsetsDialog::downloadCompleted()
         }
 
         m_userFeeds[metadata.name()] = metadata;
-        QNetworkReply *reply = download(metadata.url());
-        reply->setProperty(DocsetNameProperty, metadata.name());
-        reply->setProperty(DownloadTypeProperty, DownloadDocset);
+        Registry::Docset *docset = m_docsetRegistry->docset(metadata.name());
+        if (docset == nullptr) {
+            // Fetch docset only on first feed download,
+            // since further downloads are only update checks
+            QNetworkReply *reply = download(metadata.url());
+            reply->setProperty(DocsetNameProperty, metadata.name());
+            reply->setProperty(DownloadTypeProperty, DownloadDocset);
+        } else {
+            // Check for feed update
+            if (metadata.latestVersion() != docset->version()
+                    || metadata.revision() > docset->revision()) {
+                docset->hasUpdate = true;
+
+                if (!m_isStorageReadOnly) {
+                    ui->updateAllDocsetsButton->setEnabled(true);
+                }
+
+                ui->installedDocsetList->reset();
+            }
+        }
 
         break;
     }
 
     case DownloadDocset: {
         const QString docsetName = reply->property(DocsetNameProperty).toString();
+        const QString docsetDirectoryName = docsetName + QLatin1String(".docset");
 
-        // TODO: Implement an explicit and verbose docset update logic
-        const QDir dir(m_application->settings()->docsetPath);
-        if (dir.exists(docsetName + QLatin1String(".docset")))
+        if (QDir(m_application->settings()->docsetPath).exists(docsetDirectoryName)) {
             removeDocset(docsetName);
+        }
 
         QTemporaryFile *tmpFile = m_tmpFiles[docsetName];
         if (!tmpFile) {
@@ -439,7 +395,7 @@ void DocsetsDialog::downloadCompleted()
         }
 
         m_application->extract(tmpFile->fileName(), m_application->settings()->docsetPath,
-                               docsetName + QLatin1String(".docset"));
+                               docsetDirectoryName);
         break;
     }
     }
@@ -501,11 +457,10 @@ void DocsetsDialog::extractionCompleted(const QString &filePath)
 
     // Write metadata about docset
     Registry::DocsetMetadata metadata = m_availableDocsets.contains(docsetName)
-            ? m_availableDocsets[docsetName]
-              : m_userFeeds[docsetName];
+            ? m_availableDocsets[docsetName] : m_userFeeds[docsetName];
     metadata.save(docsetPath, metadata.latestVersion());
 
-    m_docsetRegistry->addDocset(docsetPath);
+    m_docsetRegistry->loadDocset(docsetPath);
 
     QListWidgetItem *listItem = findDocsetListItem(docsetName);
     if (listItem) {
@@ -541,6 +496,8 @@ void DocsetsDialog::extractionProgress(const QString &filePath, qint64 extracted
 
 void DocsetsDialog::loadDocsetList()
 {
+    loadUserFeedList();
+
     const QFileInfo fi(cacheLocation(DocsetListCacheFileName));
     if (!fi.exists() || fi.lastModified().msecsTo(QDateTime::currentDateTime()) > CacheTimeout) {
         downloadDocsetList();
@@ -566,6 +523,149 @@ void DocsetsDialog::loadDocsetList()
     processDocsetList(jsonDoc.array());
 }
 
+void DocsetsDialog::setupInstalledDocsetsTab()
+{
+    using Registry::ListModel;
+
+    ui->installedDocsetList->setItemDelegate(new DocsetListItemDelegate(this));
+    ui->installedDocsetList->setModel(new ListModel(m_application->docsetRegistry(), this));
+
+    if (m_isStorageReadOnly) {
+        return;
+    }
+
+    connect(ui->installedDocsetList, &QListView::activated, this, [this](const QModelIndex &index) {
+        if (!index.data(Registry::ItemDataRole::UpdateAvailableRole).toBool()) {
+            return;
+        }
+
+        downloadDashDocset(index);
+    });
+
+    QItemSelectionModel *selectionModel = ui->installedDocsetList->selectionModel();
+    connect(selectionModel, &QItemSelectionModel::selectionChanged,
+            [this, selectionModel]() {
+        ui->removeDocsetsButton->setEnabled(selectionModel->hasSelection());
+
+        for (const QModelIndex &index : selectionModel->selectedRows()) {
+            if (index.data(Registry::ItemDataRole::UpdateAvailableRole).toBool()) {
+                ui->updateSelectedDocsetsButton->setEnabled(true);
+                return;
+            }
+        }
+
+        ui->updateSelectedDocsetsButton->setEnabled(false);
+    });
+
+    connect(ui->addFeedButton, &QPushButton::clicked, this, &DocsetsDialog::addDashFeed);
+
+    connect(ui->updateSelectedDocsetsButton, &QPushButton::clicked,
+            this, &DocsetsDialog::updateSelectedDocsets);
+    connect(ui->updateAllDocsetsButton, &QPushButton::clicked,
+            this, &DocsetsDialog::updateAllDocsets);
+    connect(ui->removeDocsetsButton, &QPushButton::clicked,
+            this, &DocsetsDialog::removeSelectedDocsets);
+}
+
+void DocsetsDialog::setupAvailableDocsetsTab()
+{
+    using Registry::DocsetRegistry;
+
+    ui->availableDocsetList->setItemDelegate(new ProgressItemDelegate(this));
+
+    connect(m_docsetRegistry, &DocsetRegistry::docsetUnloaded, this, [this](const QString name) {
+        QListWidgetItem *item = findDocsetListItem(name);
+        if (!item)
+            return;
+
+        item->setHidden(false);
+    });
+    connect(m_docsetRegistry, &DocsetRegistry::docsetLoaded, this, [this](const QString name) {
+        QListWidgetItem *item = findDocsetListItem(name);
+        if (!item)
+            return;
+
+        item->setHidden(true);
+    });
+
+    connect(ui->refreshButton, &QPushButton::clicked, this, &DocsetsDialog::downloadDocsetList);
+
+    connect(ui->docsetFilterInput, &QLineEdit::textEdited,
+            this, &DocsetsDialog::updateDocsetFilter);
+
+    if (m_isStorageReadOnly) {
+        return;
+    }
+
+    connect(ui->availableDocsetList, &QListView::activated, this, [this](const QModelIndex &index) {
+        // TODO: Cancel download if it's already in progress.
+        if (index.data(ProgressItemDelegate::ShowProgressRole).toBool())
+            return;
+
+        ui->availableDocsetList->selectionModel()->select(index, QItemSelectionModel::Deselect);
+
+        QAbstractItemModel *model = ui->availableDocsetList->model();
+        model->setData(index, tr("Downloading: %p%"), ProgressItemDelegate::FormatRole);
+        model->setData(index, 0, ProgressItemDelegate::ValueRole);
+        model->setData(index, true, ProgressItemDelegate::ShowProgressRole);
+
+        downloadDashDocset(index);
+    });
+
+    QItemSelectionModel *selectionModel = ui->availableDocsetList->selectionModel();
+    connect(selectionModel, &QItemSelectionModel::selectionChanged, [this, selectionModel]() {
+        for (const QModelIndex &index : selectionModel->selectedRows()) {
+            if (!index.data(ProgressItemDelegate::ShowProgressRole).toBool()) {
+                ui->downloadDocsetsButton->setEnabled(true);
+                return;
+            }
+        }
+
+        ui->downloadDocsetsButton->setEnabled(false);
+    });
+
+    connect(ui->downloadDocsetsButton, &QPushButton::clicked,
+            this, &DocsetsDialog::downloadSelectedDocsets);
+}
+
+void DocsetsDialog::enableControls()
+{
+    // Available docsets
+    ui->refreshButton->setEnabled(true);
+
+    if (m_isStorageReadOnly) {
+        return;
+    }
+
+    // Installed docsets
+    ui->addFeedButton->setEnabled(true);
+    QItemSelectionModel *selectionModel = ui->installedDocsetList->selectionModel();
+    bool hasSelectedUpdates = false;
+    for (const QModelIndex &index : selectionModel->selectedRows()) {
+        if (index.data(Registry::ItemDataRole::UpdateAvailableRole).toBool()) {
+            hasSelectedUpdates = true;
+            break;
+        }
+    }
+
+    ui->updateSelectedDocsetsButton->setEnabled(hasSelectedUpdates);
+    ui->updateAllDocsetsButton->setEnabled(updatesAvailable());
+    ui->removeDocsetsButton->setEnabled(selectionModel->hasSelection());
+}
+
+void DocsetsDialog::disableControls()
+{
+    // Installed docsets
+    ui->addFeedButton->setEnabled(false);
+    ui->updateSelectedDocsetsButton->setEnabled(false);
+    ui->updateAllDocsetsButton->setEnabled(false);
+    ui->downloadDocsetsButton->setEnabled(false);
+    ui->removeDocsetsButton->setEnabled(false);
+
+    // Available docsets
+    ui->refreshButton->setEnabled(false);
+}
+
 QListWidgetItem *DocsetsDialog::findDocsetListItem(const QString &name) const
 {
     for (int i = 0; i < ui->availableDocsetList->count(); ++i) {
@@ -595,14 +695,7 @@ QNetworkReply *DocsetsDialog::download(const QUrl &url)
     connect(reply, &QNetworkReply::finished, this, &DocsetsDialog::downloadCompleted);
     m_replies.append(reply);
 
-    // Installed docsets
-    ui->addFeedButton->setEnabled(false);
-    ui->updateSelectedDocsetsButton->setEnabled(false);
-    ui->updateAllDocsetsButton->setEnabled(false);
-    ui->removeDocsetsButton->setEnabled(false);
-
-    // Available docsets
-    ui->refreshButton->setEnabled(false);
+    disableControls();
 
     updateCombinedProgress();
 
@@ -627,6 +720,16 @@ void DocsetsDialog::cancelDownloads()
     resetProgress();
 }
 
+void DocsetsDialog::loadUserFeedList()
+{
+    for (Registry::Docset *docset : m_docsetRegistry->docsets()) {
+        if (!docset->feedUrl().isEmpty()) {
+            QNetworkReply *reply = download(QUrl(docset->feedUrl()));
+            reply->setProperty(DownloadTypeProperty, DownloadDashFeed);
+        }
+    }
+}
+
 void DocsetsDialog::downloadDocsetList()
 {
     ui->availableDocsetList->clear();
@@ -659,7 +762,10 @@ void DocsetsDialog::processDocsetList(const QJsonArray &list)
             if (metadata.latestVersion() != docset->version()
                     || metadata.revision() > docset->revision()) {
                 docset->hasUpdate = true;
-                ui->updateAllDocsetsButton->setEnabled(true);
+
+                if (!m_isStorageReadOnly) {
+                    ui->updateAllDocsetsButton->setEnabled(true);
+                }
             }
         }
     }
@@ -671,11 +777,19 @@ void DocsetsDialog::downloadDashDocset(const QModelIndex &index)
 {
     const QString name = index.data(Registry::ItemDataRole::DocsetNameRole).toString();
 
-    if (!m_availableDocsets.contains(name))
+    if (!m_availableDocsets.contains(name) && !m_userFeeds.contains(name))
         return;
 
-    const QString urlString = RedirectServerUrl + QStringLiteral("/d/com.kapeli/%1/latest");
-    QNetworkReply *reply = download(QUrl(urlString.arg(name)));
+    QUrl url;
+    if (!m_userFeeds.contains(name)) {
+        // No feed present means that this is a Kapeli docset
+        QString urlString = RedirectServerUrl + QString("/d/com.kapeli/%1/latest");
+        url = QUrl(urlString.arg(name));
+    } else {
+        url = m_userFeeds[name].url();
+    }
+
+    QNetworkReply *reply = download(url);
     reply->setProperty(DocsetNameProperty, name);
     reply->setProperty(DownloadTypeProperty, DownloadDocset);
     reply->setProperty(ListItemIndexProperty,
@@ -684,50 +798,23 @@ void DocsetsDialog::downloadDashDocset(const QModelIndex &index)
 
 void DocsetsDialog::removeDocset(const QString &name)
 {
-    if (m_docsetsBeingDeleted.contains(name))
-        return;
+    if (m_docsetRegistry->contains(name)) {
+        m_docsetRegistry->unloadDocset(name);
+    }
 
-    m_docsetsBeingDeleted.append(name);
-
-    Registry::Docset *docset = m_docsetRegistry->docset(name);
-    const QString title = docset->title();
-    const QString docsetPath = docset->path();
-    const QString tmpPath = docsetPath + QLatin1String(".deleteme.")
-            + QString::number(QDateTime::currentMSecsSinceEpoch());
-
-    // Remove from registry first to avoid renaming files in use on Windows.
-    m_docsetRegistry->remove(name);
-
-    // Rename first to allow simultaneous installation.
-    if (!QDir().rename(docsetPath, tmpPath)) {
-        const QString error = tr("Cannot delete docset <b>%1</b>! Please try closing other "
-                                 "applications first, as they may be accessing the docset "
-                                 "files.").arg(title);
+    const QString docsetPath
+            = QDir(m_application->settings()->docsetPath).filePath(name + QLatin1String(".docset"));
+    if (!m_application->fileManager()->removeRecursively(docsetPath)) {
+        const QString error = tr("Cannot remove directory <b>%1</b>! It might be in use"
+                                 " by another process.").arg(docsetPath);
         QMessageBox::warning(this, QStringLiteral("Zeal"), error);
-        m_docsetsBeingDeleted.removeOne(name);
-        m_docsetRegistry->addDocset(docsetPath);
         return;
     }
 
-    QFutureWatcher<bool> *watcher = new QFutureWatcher<bool>();
-    connect(watcher, &QFutureWatcher<void>::finished, [=] {
-        if (!watcher->result()) {
-            QMessageBox::warning(this, QStringLiteral("Zeal"),
-                                 tr("Cannot delete docset <b>%1</b>!").arg(title));
-        }
-
-        QListWidgetItem *listItem = findDocsetListItem(name);
-        if (listItem)
-            listItem->setHidden(false);
-
-        watcher->deleteLater();
-
-        m_docsetsBeingDeleted.removeOne(name);
-    });
-
-    watcher->setFuture(QtConcurrent::run([tmpPath] {
-        return QDir(tmpPath).removeRecursively();
-    }));
+    QListWidgetItem *listItem = findDocsetListItem(name);
+    if (listItem) {
+        listItem->setHidden(false);
+    }
 }
 
 void DocsetsDialog::updateCombinedProgress()
@@ -754,23 +841,7 @@ void DocsetsDialog::resetProgress()
     m_combinedReceived = 0;
     m_combinedTotal = 0;
 
-    // Installed docsets
-    ui->addFeedButton->setEnabled(true);
-    QItemSelectionModel *selectionModel = ui->installedDocsetList->selectionModel();
-    bool hasSelectedUpdates = false;
-    for (const QModelIndex &index : selectionModel->selectedRows()) {
-        if (index.data(Registry::ItemDataRole::UpdateAvailableRole).toBool()) {
-            hasSelectedUpdates = true;
-            break;
-        }
-    }
-
-    ui->updateSelectedDocsetsButton->setEnabled(hasSelectedUpdates);
-    ui->updateAllDocsetsButton->setEnabled(updatesAvailable());
-    ui->removeDocsetsButton->setEnabled(selectionModel->hasSelection());
-
-    // Available docsets
-    ui->refreshButton->setEnabled(true);
+    enableControls();
 }
 
 QString DocsetsDialog::docsetNameForTmpFilePath(const QString &filePath) const
@@ -794,13 +865,5 @@ int DocsetsDialog::percent(qint64 fraction, qint64 total)
 
 QString DocsetsDialog::cacheLocation(const QString &fileName)
 {
-#ifndef PORTABLE_BUILD
-    const QDir cacheDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
-#else
-    const QDir cacheDir(QCoreApplication::applicationDirPath() + QLatin1String("/cache"));
-#endif
-    // TODO: Report error
-    QDir().mkpath(cacheDir.path());
-
-    return cacheDir.filePath(fileName);
+    return QDir(Core::FileManager::cacheLocation()).filePath(fileName);
 }
