@@ -24,8 +24,6 @@
 #ifndef ZEAL_WIDGETUI_MAINWINDOW_H
 #define ZEAL_WIDGETUI_MAINWINDOW_H
 
-#include <registry/searchquery.h>
-
 #include <QMainWindow>
 
 class QxtGlobalShortcut;
@@ -44,6 +42,7 @@ class Settings;
 
 namespace Registry {
 class ListModel;
+class SearchQuery;
 } //namespace Registry
 
 namespace WidgetUi {
@@ -52,18 +51,21 @@ namespace Ui {
 class MainWindow;
 } // namespace Ui
 
+class WebBridge;
+class WebViewTab;
+
 struct TabState;
 
 class MainWindow : public QMainWindow
 {
     Q_OBJECT
 public:
-    explicit MainWindow(Zeal::Core::Application *app, QWidget *parent = nullptr);
+    explicit MainWindow(Core::Application *app, QWidget *parent = nullptr);
     ~MainWindow() override;
 
-    void search(const Zeal::Registry::SearchQuery &query);
+    void search(const Registry::SearchQuery &query);
     void bringToFront();
-    void createTab(int index = -1);
+    WebViewTab *createTab(int index = -1);
 
 public slots:
     void toggleWindow();
@@ -79,6 +81,7 @@ private slots:
     void openDocset(const QModelIndex &index);
     void queryCompleted();
     void closeTab(int index = -1);
+    void moveTab(int from, int to);
     void duplicateTab(int index);
 
 private:
@@ -88,6 +91,10 @@ private:
     void setupTabBar();
 
     TabState *currentTabState() const;
+    WebViewTab *currentTab() const;
+
+    void attachTab(TabState *tabState);
+    void detachTab(TabState *tabState);
 
     QString docsetName(const QUrl &url) const;
     QIcon docsetIcon(const QString &docsetName) const;
@@ -95,12 +102,16 @@ private:
     void createTrayIcon();
     void removeTrayIcon();
 
+    void syncTabState(TabState *tabState);
+
     QList<TabState *> m_tabStates;
 
     Ui::MainWindow *ui = nullptr;
-    Zeal::Core::Application *m_application = nullptr;
-    Zeal::Core::Settings *m_settings = nullptr;
-    Zeal::Registry::ListModel *m_zealListModel = nullptr;
+    Core::Application *m_application = nullptr;
+    Core::Settings *m_settings = nullptr;
+    Registry::ListModel *m_zealListModel = nullptr;
+
+    WebBridge *m_webBridge = nullptr;
 
     QMenu *m_backMenu = nullptr;
     QMenu *m_forwardMenu = nullptr;
