@@ -24,6 +24,8 @@
 #ifndef ZEAL_WIDGETUI_WEBVIEW_H
 #define ZEAL_WIDGETUI_WEBVIEW_H
 
+#include <QVector>
+#include <QWebFrame>
 #include <QWebView>
 
 namespace Zeal {
@@ -35,21 +37,35 @@ class WebView : public QWebView
 public:
     explicit WebView(QWidget *parent = nullptr);
 
-    int zealZoomFactor() const;
-    void setZealZoomFactor(int zf);
+    int zoomLevel() const;
+    void setZoomLevel(int level);
+
+    static const QVector<int> &availableZoomLevels();
+    static const int defaultZoomLevel();
+
+public slots:
+    void zoomIn();
+    void zoomOut();
+    void resetZoom();
+
+signals:
+    void zoomLevelChanged();
 
 protected:
     QWebView *createWindow(QWebPage::WebWindowType type) override;
+    void contextMenuEvent(QContextMenuEvent *event) override;
     void mousePressEvent(QMouseEvent *event) override;
     void mouseReleaseEvent(QMouseEvent *event) override;
     void wheelEvent(QWheelEvent *event) override;
 
 private:
-    QUrl clickedLink(const QPoint &pos) const;
-    void updateZoomFactor();
+    QWebHitTestResult hitTestContent(const QPoint &pos) const;
 
+    static bool isUrlExternal(const QUrl &url);
+
+    QMenu *m_contextMenu = nullptr;
     QUrl m_clickedLink;
-    int m_zoomFactor = 0;
+    int m_zoomLevel = defaultZoomLevel();
 };
 
 } // namespace WidgetUi
